<?php if (!defined('PmWiki')) exit();
/*
cmsmode.php - Adds some CMS Mode features for PmWiki
Copyright 2006,2012,2014,2016 by Hagan Fox
This program is distributed under the terms of the
GNU General Public License, Version 2

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License, Version 2 as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/
SDV($CMSVersion, '2016-09-28');

## Set RecipeInfo and create {$CMSVersion} page variable
SDV($RecipeInfo['CMSMode']['Version'], $CMSVersion);
SDV($FmtPV['$CMSVersion'],'$CMSVersion');

if (CondAuth($SiteGroup.'.'.$SiteGroup, 'admin')) return; # return if admin..
SDV($EnableCMSMode, TRUE); if ($EnableCMSMode == FALSE) return; #..or disabled

## TODO Sidebar changes when ?action=search for authors ?

/**
*  Settings
*/
## Consider the visitor an author if they're authorized to edit this page.
SDV($CMSEditTestPage, $DefaultGroup.'.WikiSandbox');
#
## Whitelisted  wikiactions are allowed for non-authors.
SDVA($CMSAllowedActions,array(
  'browse' => 'browse', 'print' => 'print', 'search' => 'search',
  'setprefs' => 'setprefs', 'login' => 'login', 'logout' => 'logout',
  'edit' => 'edit', 'rss' => 'rss', 'atom' => 'atom',
  'approvesites' => 'approvesites', 'skin' => 'skin',
  'setskin' => 'setskin', 'addlink' => 'addlink', 'comment' => 'comment'));
#
SDV($EnableDraftRedir, TRUE); // Non-authors can't see drafts (-Draft pages).
SDV($EnableTalkRedir, TRUE); // Non-authors can't see -Talk discussion pages.
#
## Authors can visit these Redirect Groups and Pages, non-authors can't.
# TODO 301? http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html#sec10.3.2
SDVA($CMSRedirGroups,array(
  'Test' => 'Test',
  'PmWiki' => 'PmWiki',
  'Profiles' => 'Profiles'));
SDVA($CMSRedirNames,array(
  'GroupAttributes' => 'GroupAttributes',
  'WikiSandbox' => 'WikiSandbox',
  'RecentDraftChanges' => 'RecentDraftChanges'));
#
## These groups and pages are excluded from search results.
SDVA($CMSExclSearchGroups,array(
  'Temp' => 'Temp',
  'Test' => 'Test',
  'PmWiki' => 'PmWiki',
  'Profiles' => 'Profiles'));
SDVA($CMSExclSearchNames,array(
  'GroupAttributes' => 'GroupAttributes',
  'WikiSandbox' => 'WikiSandbox'));
#
## Maintain a special "public" RSS/Atom feed changelog page.
SDV($FeedAllRecentChanges, 'Main.AllRecentChanges');
## group and page names excluded from public A.R.C page.
SDVA($CMSExclARCGroups,array(
  'Temp' => 'Temp',
  'Test' => 'Test',
  'PmWiki' => 'PmWiki',
  'Profiles' => 'Profiles'));
SDVA($CMSExclARCNames,array(
  'WikiSandbox' => 'WikiSandbox',
  'Search' => 'Search'));
#
#
## Show how many RecentChanges links in masthead? (4=just show GroupRC)
## TODO: XL page for link text.
SDV($CMSHeaderShowRC,1);
if    ($CMSHeaderShowRC==0) { $CMSrc='.grouprc, .allrc,'; } # hide both
elseif($CMSHeaderShowRC==1) { $CMSrc='.grouprc,'; } # hide GroupRC
elseif($CMSHeaderShowRC==2) { $CMSrc=''; } # show both
elseif($CMSHeaderShowRC==4) { $CMSrc='.allrc,'; } # hide AllRC
#
## Add some CMS Mode styles using $HTMLStylesFmt[] array variable.
SDV($CMSModeStylesPmSkin, // PmWiki Skin 
  "  #wikihead a { text-decoration:none; color:black;font-size:1.1em; }
  #wikihead a:hover { text-decoration:underline; color:blue;font-size:1.1em;  }
  .pagegroup, .headnav, .footnav { display:none; }
  .pagetitle { margin-top:10px; } ");
SDV($CMSModeStyles, // Custom skin, e.g. Adapt Skin
  "  #wikihead a { text-decoration:none; color:black;font-size:1.1em; }
  #wikihead a:hover { text-decoration:underline; color:blue;font-size:1.1em;  }
 .pagetitle { margin-top:8px; } #wikititle { margin-top:8px; margin-bottom:2px; }
  #wikicmds, .pagegroup, .pagegroup2, .grouprc, .allrc, .footnav  { display:none; }
 .lastmod { color:#999999; }\n");
#
## Set Sensible Defaults(TM) for authors on read-only pages e.g. Site.Search.
SDV($HandleAuth['source'], 'edit'); // Only see source of an editable page
SDV($HandleAuth['diff'], 'edit'); // See diff only if page is editable
#SDV($HandleAuth['upload'], 'edit'); // Can't edit? Can't upload.
#
SDV($CMSTimeFormat,'%B %d, %Y'); // Time format for last-updated notice
#
## Allowed values for the the {$View} page action (?view=<allowed value>)
SDVA($CMSViewAllow,array(
  'site' => 'site',
  'wiki'  => 'wiki',
  'alt' => 'alt'));


/**
*  Logic
*/
## Get the group and page names. (PageVar() caused unexpected results.)
$CMSpage = explode(".", $pagename);
$CMSgroup = $CMSpage[0]; // same as {$Group}
$CMSname = $CMSpage[1]; // same as {$Name}

## Create the {$View} page action only for allowed ?view= requests
if (isset($_REQUEST['view']) && in_array($_REQUEST['view'], $CMSViewAllow))
  { $FmtPV['$View'] = '$_REQUEST["view"]'; $view=$_REQUEST['view']; }

## Customize (All)RecentChanges pages for better web feeds.
if (preg_match('!^'."($SiteGroup|$SiteAdminGroup)".'\\.!', $pagename)
  || in_array($CMSgroup,$CMSExclARCGroups)
  || preg_match('!\\.(GroupHeader|GroupFooter|GroupAttributes|'
    .'RecentDraftChanges|AllRecentChanges)$!', $pagename)
  || in_array($CMSname,$CMSExclARCNames)) {
  $RecentChangesFmt['$Group.RecentChanges'] = '';
  $RecentChangesFmt[$FeedAllRecentChanges] = '';
} else {
  $RecentChangesFmt[$FeedAllRecentChanges] =
    '* [[{$Group}.{$Name}]]  $[was modified] $CurrentTime. [=$ChangeSummary=]';
}

##  Keep a separate RecentDraftChanges page for drafts.
# TODO: Experimental
$DraftRecentChangesFmt['$Group.RecentDraftChanges'] =
  '* [[{$Group}/{$Name}]]  . . . $CurrentTime $[by] $AuthorLink: [=$ChangeSummary=]';
#$DraftRecentChangesFmt['$Group.RecentChanges'] = '';

## Crate a {$BaseName} page variable for "-Talk" and "-Draft" pages.
## TODO: Move this?
if (preg_match('/-Talk$/', $pagename)) $BaseNamePatterns['/-Talk$/'] = '';
if (preg_match('/-Draft$/', $pagename)) $BaseNamePatterns['/-Draft$/'] = '';


## For searches and wikiactions we check the Test Page to determine
## author/non-author, not the current page because e.g. Site.Search is read-only
#
## Unset CSS styles if a visitor requests wiki view with e.g. ?view=wiki
if (@$view == 'wiki') {
 $CMSModeStylesPmSkin = "/* in wiki vew */\n";
 $CMSModeStyles = "/* in wiki vew */\n";
}
if (! CondAuth($CMSEditTestPage, 'edit') || $view == 'site') {
  ## Exclude Certain pages / groups from search results.
  $SearchPatterns['default'][] = '!\\.(All)?Recent(DraftChanges|Changes|Uploads)$!';
  $SearchPatterns['default'][] = '!\\.(Group|Print)(Header|Footer)$!';
  $SearchPatterns['default'][] = '!^'."($SiteGroup|$SiteAdminGroup)".'\\.!';
  $CMSexsg = implode('|',$CMSExclSearchGroups);
  $SearchPatterns['default'][] = '!^('.$CMSexsg.')\\.!';
  $CMSexsn = implode('|',$CMSExclSearchNames);
  $SearchPatterns['default'][] = '!\\.('.$CMSexsn.')$!';
  if (IsEnabled($EnableDraftRedir)) $SearchPatterns['default'][] = '!-Draft$!';
  if (IsEnabled($EnableTalkRedir)) $SearchPatterns['default'][] = '!-Talk$!';
  if ($pagename == $SiteGroup.'.AllRecentChanges') Redirect($FeedAllRecentChanges);
  ## Turn off wikiactions except when a visitor requests wiki view
  if (! @$view == 'wiki') SetTmplDisplay('PageActionFmt', 0);
  ## Inject some CSS for the default skin or a custom skin (e.g. Adapt Skin).
  if ($GLOBALS[$Skin] == 'pmwiki') {
    SDV($HTMLStylesFmt['cmsmode_pm'], $CMSModeStylesPmSkin);
  } else {
    $HTMLStylesFmt['cmsmode'] = $CMSModeStyles;
  }
  ## Format the time string for last-updated notices.
  $TimeFormat = $CMSTimeFmt;
}

## Create some special restrictions for read-only pages, which is all or
## nearly all pages if there's a site-wide edit password.
if (! CondAuth($pagename, 'edit')) {
  ##  Allow only essential site-related actions.
  if (! in_array($action, $CMSAllowedActions)) $action='browse';
  ## Deny access to certain groups / pages.
  if (in_array($CMSgroup, $CMSRedirGroups)
    || (preg_match('!\\.('.$SiteGroup.'|'.$SiteAdminGroup.')$!', $pagename)
      && !preg_match('!\\.(Search|ActionLog|AllRecentChanges)$!', $pagename)) // TODO Setting?
    || preg_match('!\\.(GroupHeader|GroupFooter|GroupAttributes|'
         .'AuthUser|RecentDraftChanges)$!', $pagename)
    || in_array($CMSname, $CMSRedirNames)) {
    Redirect($DefaultPage);
  }
  ## Stealth draft pages // TODO: $DraftSuffix
  if (IsEnabled($EnableDraftRedir)) {
    $basename = preg_replace("/-.*\$/", '', $pagename);
    if (preg_match('/-Draft$/', $pagename)) Redirect($basename);
  }
  ## Stealth discussion pages
  if (IsEnabled($EnableTalkRedir)) { 
    $basename = preg_replace("/-.*\$/", '', $pagename);
    if (preg_match('/-Talk$/', $pagename)) Redirect($basename);
  }
  ## Lose the rel='nofollow' attribute for searchable external links.
  $UrlLinkFmt = "<a class='urllink' href='\$LinkUrl' title='\$LinkAlt'>\$LinkText</a>";
  if (in_array($CMSgroup,$CMSExclSearchGroups))
    $UrlLinkFmt = "<a class='urllink' href='\$LinkUrl' title='\$LinkAlt' rel='nofollow'>\$LinkText</a>";
}
